package com.progress.fuse.se.camel;

import java.util.ArrayList;
import java.util.List;

import org.apache.camel.Exchange;
import org.apache.camel.ExchangePattern;
import org.apache.camel.Message;
import org.apache.camel.Processor;
import org.apache.camel.ProducerTemplate;
import org.apache.camel.builder.RouteBuilder;
import org.apache.camel.builder.xml.Namespaces;
import org.apache.camel.converter.jaxb.JaxbDataFormat;
import org.apache.camel.spi.DataFormat;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.fusesource.camel_webinar.personservice.*;

public class MyRouteBuilder extends RouteBuilder {
    private static final Log LOG = LogFactory.getLog(MyRouteBuilder.class);

    @Override
    public void configure() {
        Namespaces namespace = new Namespaces("per", "http://fusesource.com/camel_webinar/PersonService/");
        
        // Used to marshal and unmarshal XML messages to/from the CXF endpoint
        DataFormat jaxb = new JaxbDataFormat("com.fusesource.camel_webinar.personservice");
        
        //trigger route based on two second timer
        from("timer://myTimer?fixedRate=true&period=2000").
        process(new Processor() {
        	public void process(Exchange exchange) throws Exception {
        		com.fusesource.camel_webinar.personservice.Person person
            		= new Person();
	            person.setCity("London");
	            person.setFirstName("James");
	            person.setLastName("Strachan");
	            person.setUser("jstrachan"); 
	            
	            ProducerTemplate template = exchange.getContext().createProducerTemplate();
	            template.sendBody("seda:requestTrigger", person);
        	}
        });
        
        //process the web service request and invoke the web service
        from("seda:requestTrigger").
        process(new Processor() {
        	public void process(Exchange exchange) throws Exception {
        		final List params = new ArrayList();
                
        		Message msg = exchange.getIn();
        		params.add(msg.getBody());
        		msg.setBody(params);
        		msg.setHeader("operationName", "ProcessPerson");
        		exchange.setIn(msg);
        	}
        }).
        to("log:webServiceRequestLog").
        to("cxf:bean:PersonEndpoint").
        process(new Processor() {
        	public void process(Exchange exchange) throws Exception {
        		Message msg = exchange.getIn();
        		List params = (List)msg.getBody();
        		if(params.size() > 0)
        		{
        			com.fusesource.camel_webinar.personservice.PersonResponse response = 
        				(com.fusesource.camel_webinar.personservice.PersonResponse)params.get(0);
        			msg.setBody(response);
        			exchange.setIn(msg);
        		}
        	}
        }).
        marshal(jaxb).
        convertBodyTo(String.class).
        to("log:webServiceResponseLog");

        
// WEB SERVICE DEFINITION BELOW, COULD ALSO BE EXTERNAL WEB SERVICE //        
        // Prepare a Response object to be sent back to the caller after the
        // message is sent to the JMS Queue
        PersonResponse response = new PersonResponse();
        response.setAcknowledgement("OK");

        // This Route will create a Jetty based HTTP Listener for the CXF
        // endpoint, Transform the Request to the XML form the CBR route is
        // expecting, and then send a Response to the WS Caller
        from("cxf:bean:PersonEndpoint").
        convertBodyTo(Person.class).
        marshal(jaxb).
        convertBodyTo(String.class).
        process(new Processor() {

            // This Processor step isn't required, and is only here to
            // highlight how you could insert custom code into the Processor
            // Chain
            public void process(Exchange exchange) throws Exception {
                Object msg = exchange.getIn().getBody();

                LOG.info("Custom Processor : msg = " + msg.toString());
            }

        }).
        setBody(constant(response));

    }
}
